@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2015, Met Office
+# (C) British Crown Copyright 2010 - 2016, Met Office
 #
 # This file is part of Iris.
 #
@@ -31,6 +31,7 @@ import six
 import collections
 import os
 import os.path
+import re
 import string
 import warnings
 
@@ -40,6 +41,7 @@ import numpy as np
 import numpy.ma as ma
 from pyke import knowledge_engine
 
+from iris._deprecation import warn_deprecated
 import iris.analysis
 from iris.aux_factory import HybridHeightFactory, HybridPressureFactory, \
     OceanSigmaZFactory, OceanSigmaFactory, OceanSFactory, OceanSg1Factory, \
@@ -82,7 +84,7 @@ _CF_ATTRS = ['add_offset', 'ancillary_variables', 'axis', 'bounds', 'calendar',
              'coordinates', '_FillValue', 'formula_terms', 'grid_mapping',
              'leap_month', 'leap_year', 'long_name', 'missing_value',
              'month_lengths', 'scale_factor', 'standard_error_multiplier',
-             'standard_name', 'units', 'valid_max', 'valid_min', 'valid_range']
+             'standard_name', 'units']
 
 # CF attributes that should not be global.
 _CF_DATA_ATTRS = ['flag_masks', 'flag_meanings', 'flag_values',
@@ -136,6 +138,124 @@ _FACTORY_DEFNS = {
 }
 
 
+# Cell methods.
+_CM_KNOWN_METHODS = ['point', 'sum', 'mean', 'maximum', 'minimum', 'mid_range',
+                     'standard_deviation', 'variance', 'mode', 'median']
+
+_CM_COMMENT = 'comment'
+_CM_EXTRA = 'extra'
+_CM_INTERVAL = 'interval'
+_CM_METHOD = 'method'
+_CM_NAME = 'name'
+_CM_PARSE = re.compile(r'''
+                           (?P<name>([\w_]+\s*?:\s+)+)
+                           (?P<method>[\w_\s]+(?![\w_]*\s*?:))\s*
+                           (?:
+                               \(\s*
+                               (?P<extra>[^\)]+)
+                               \)\s*
+                           )?
+                       ''', re.VERBOSE)
+
+
+class UnknownCellMethodWarning(Warning):
+    pass
+
+
+def parse_cell_methods(nc_cell_methods):
+    """
+    Parse a CF cell_methods attribute string into a tuple of zero or
+    more CellMethod instances.
+
+    Args:
+
+    * nc_cell_methods (str):
+        The value of the cell methods attribute to be parsed.
+
+    Returns:
+
+    * cell_methods
+        An iterable of :class:`iris.coords.CellMethod`.
+
+    Multiple coordinates, intervals and comments are supported.
+    If a method has a non-standard name a warning will be issued, but the
+    results are not affected.
+
+    """
+
+    cell_methods = []
+    if nc_cell_methods is not None:
+        for m in _CM_PARSE.finditer(nc_cell_methods):
+            d = m.groupdict()
+            method = d[_CM_METHOD]
+            method = method.strip()
+            # Check validity of method, allowing for multi-part methods
+            # e.g. mean over years.
+            method_words = method.split()
+            if method_words[0].lower() not in _CM_KNOWN_METHODS:
+                msg = 'NetCDF variable contains unknown cell method {!r}'
+                warnings.warn(msg.format('{}'.format(method_words[0])),
+                              UnknownCellMethodWarning)
+            d[_CM_METHOD] = method
+            name = d[_CM_NAME]
+            name = name.replace(' ', '')
+            name = name.rstrip(':')
+            d[_CM_NAME] = tuple([n for n in name.split(':')])
+            interval = []
+            comment = []
+            if d[_CM_EXTRA] is not None:
+                #
+                # tokenise the key words and field colon marker
+                #
+                d[_CM_EXTRA] = d[_CM_EXTRA].replace('comment:',
+                                                    '<<comment>><<:>>')
+                d[_CM_EXTRA] = d[_CM_EXTRA].replace('interval:',
+                                                    '<<interval>><<:>>')
+                d[_CM_EXTRA] = d[_CM_EXTRA].split('<<:>>')
+                if len(d[_CM_EXTRA]) == 1:
+                    comment.extend(d[_CM_EXTRA])
+                else:
+                    next_field_type = comment
+                    for field in d[_CM_EXTRA]:
+                        field_type = next_field_type
+                        index = field.rfind('<<interval>>')
+                        if index == 0:
+                            next_field_type = interval
+                            continue
+                        elif index > 0:
+                            next_field_type = interval
+                        else:
+                            index = field.rfind('<<comment>>')
+                            if index == 0:
+                                next_field_type = comment
+                                continue
+                            elif index > 0:
+                                next_field_type = comment
+                        if index != -1:
+                            field = field[:index]
+                        field_type.append(field.strip())
+            #
+            # cater for a shared interval over multiple axes
+            #
+            if len(interval):
+                if len(d[_CM_NAME]) != len(interval) and len(interval) == 1:
+                    interval = interval*len(d[_CM_NAME])
+            #
+            # cater for a shared comment over multiple axes
+            #
+            if len(comment):
+                if len(d[_CM_NAME]) != len(comment) and len(comment) == 1:
+                    comment = comment*len(d[_CM_NAME])
+            d[_CM_INTERVAL] = tuple(interval)
+            d[_CM_COMMENT] = tuple(comment)
+            cell_method = iris.coords.CellMethod(d[_CM_METHOD],
+                                                 coords=d[_CM_NAME],
+                                                 intervals=d[_CM_INTERVAL],
+                                                 comments=d[_CM_COMMENT])
+            cell_methods.append(cell_method)
+    return tuple(cell_methods)
+
+
 class CFNameCoordMap(object):
     """Provide a simple CF name to CF coordinate mapping."""
 
@@ -302,6 +422,11 @@ def _assert_case_specific_facts(engine, cf, cf_group):
         engine.add_case_specific_fact(_PYKE_FACT_BASE, 'auxiliary_coordinate',
                                       (cf_name,))
 
+    # Assert facts for CF cell measures.
+    for cf_name in six.iterkeys(cf_group.cell_measures):
+        engine.add_case_specific_fact(_PYKE_FACT_BASE,
+                                      'cell_measure', (cf_name,))
+
     # Assert facts for CF grid_mappings.
     for cf_name in six.iterkeys(cf_group.grid_mappings):
         engine.add_case_specific_fact(_PYKE_FACT_BASE, 'grid_mapping',
@@ -414,6 +539,19 @@ def _load_cube(engine, cf, cf_var, filename):
     for attr_name, attr_value in tmpvar:
         _set_attributes(cube.attributes, attr_name, attr_value)
 
+    names = {coord.var_name: coord.standard_name or coord.var_name or 'unknown'
+             for coord in cube.coords()}
+
+    cube.cell_methods = [
+        iris.coords.CellMethod(
+            method=method.method,
+            intervals=method.intervals,
+            comments=method.comments,
+            coords=[names[coord_name] if
+                    coord_name in names else coord_name
+                    for coord_name in method.coord_names])
+        for method in cube.cell_methods]
+
     # Show pyke session statistics.
     _pyke_stats(engine, cf_var.cf_name)
 
@@ -747,6 +885,12 @@ class Saver(object):
             # being raised if mandatory requirements are not satisfied.
             profile = iris.site_configuration['cf_profile'](cube)
 
+        # Ensure that attributes are CF compliant and if possible to make them
+        # compliant.
+        self.check_attribute_compliance(cube, cube.lazy_data())
+        for coord in cube.coords():
+            self.check_attribute_compliance(coord, coord.points)
+
         # Get suitable dimension names.
         dimension_names = self._get_dim_names(cube)
 
@@ -767,6 +911,10 @@ class Saver(object):
         # variable to them
         self._add_aux_coords(cube, cf_var_cube, dimension_names)
 
+        # Add the cell_measures variable names and associate the data
+        # variable to them
+        self._add_cell_measures(cube, cf_var_cube, dimension_names)
+
         # Add the formula terms to the appropriate cf variables for each
         # aux factory in the cube.
         self._add_aux_factories(cube, cf_var_cube, dimension_names)
@@ -794,6 +942,37 @@ class Saver(object):
                     'cf_patch')
                 warnings.warn(msg)
 
+    @staticmethod
+    def check_attribute_compliance(container, data):
+        def _coerce_value(val_attr, val_attr_value, data_dtype):
+            val_attr_tmp = np.array(val_attr_value, dtype=data_dtype)
+            if (val_attr_tmp != val_attr_value).any():
+                msg = '"{}" is not of a suitable value ({})'
+                raise ValueError(msg.format(val_attr, val_attr_value))
+            return val_attr_tmp
+
+        data_dtype = data.dtype
+
+        # Ensure that conflicting attributes are not provided.
+        if ((container.attributes.get('valid_min') is not None or
+                container.attributes.get('valid_max') is not None) and
+                container.attributes.get('valid_range') is not None):
+            msg = ('Both "valid_range" and "valid_min" or "valid_max" '
+                   'attributes present.')
+            raise ValueError(msg)
+
+        # Ensure correct datatype
+        for val_attr in ['valid_range', 'valid_min', 'valid_max']:
+            val_attr_value = container.attributes.get(val_attr)
+            if val_attr_value is not None:
+                val_attr_value = np.asarray(val_attr_value)
+                if data_dtype.itemsize is 1:
+                    # Allow signed integral type
+                    if val_attr_value.dtype.kind == 'i':
+                        continue
+                new_val = _coerce_value(val_attr, val_attr_value, data_dtype)
+                container.attributes[val_attr] = new_val
+
     def update_global_attributes(self, attributes=None, **kwargs):
         """
         Update the CF global attributes based on the provided
@@ -896,6 +1075,41 @@ class Saver(object):
             cf_var_cube.coordinates = ' '.join(
                 sorted(auxiliary_coordinate_names))
 
+    def _add_cell_measures(self, cube, cf_var_cube, dim_names):
+        """
+        Add cell measures to the dataset and associate with the data variable
+
+        Args:
+
+        * cube (:class:`iris.cube.Cube`):
+            A :class:`iris.cube.Cube` to be saved to a netCDF file.
+        * cf_var_cube (:class:`netcdf.netcdf_variable`):
+            cf variable cube representation.
+        * dimension_names (list):
+            Names associated with the dimensions of the cube.
+
+        """
+        cell_measure_names = []
+        # Add CF-netCDF variables for the associated cell measures.
+        for cm in sorted(cube.cell_measures(), key=lambda cm: cm.name()):
+            # Create the associated cell measure CF-netCDF variable.
+            if cm not in self._name_coord_map.coords:
+                cf_name = self._create_cf_cell_measure_variable(cube,
+                                                                dim_names,
+                                                                cm)
+                self._name_coord_map.append(cf_name, cm)
+            else:
+                cf_name = self._name_coord_map.name(cm)
+
+            if cf_name is not None:
+                cell_measure_names.append('{}: {}'.format(cm.measure, cf_name))
+
+        # Add CF-netCDF cell measure variable references to the
+        # CF-netCDF data variable.
+        if cell_measure_names:
+            cf_var_cube.cell_measures = ' '.join(
+                sorted(cell_measure_names))
+
     def _add_dim_coords(self, cube, dimension_names):
         """
         Add coordinate variables to NetCDF dataset.
@@ -1074,21 +1288,17 @@ class Saver(object):
         """
 
         units = str(coord.units)
-
-        # TODO: Use #61 to get the units.
-        if isinstance(coord.coord_system, iris.coord_systems.GeogCS):
-            if "latitude" in coord.standard_name:
+        # Set the 'units' of 'latitude' and 'longitude' coordinates specified
+        # in 'degrees' to 'degrees_north' and 'degrees_east' respectively,
+        # as defined in the CF conventions for netCDF files: sections 4.1 and
+        # 4.2.
+        if ((isinstance(coord.coord_system, iris.coord_systems.GeogCS) or
+                coord.coord_system is None) and coord.units == 'degrees'):
+            if coord.standard_name == "latitude":
                 units = 'degrees_north'
-            elif "longitude" in coord.standard_name:
+            elif coord.standard_name == "longitude":
                 units = 'degrees_east'
 
-        elif isinstance(coord.coord_system, iris.coord_systems.RotatedGeogCS):
-            units = 'degrees'
-
-        elif isinstance(coord.coord_system,
-                        iris.coord_systems.TransverseMercator):
-            units = 'm'
-
         return coord.standard_name, coord.long_name, units
 
     def _ensure_valid_dtype(self, values, src_name, src_object):
@@ -1204,6 +1414,63 @@ class Saver(object):
 
         return cf_name
 
+    def _create_cf_cell_measure_variable(self, cube, dimension_names,
+                                         cell_measure):
+        """
+        Create the associated CF-netCDF variable in the netCDF dataset for the
+        given cell_measure.
+
+        Args:
+
+        * cube (:class:`iris.cube.Cube`):
+            The associated cube being saved to CF-netCDF file.
+        * dimension_names (list):
+            Names for each dimension of the cube.
+        * cell_measure (:class:`iris.coords.CellMeasure`):
+            The cell measure to be saved to CF-netCDF file.
+
+        Returns:
+            The string name of the associated CF-netCDF variable saved.
+
+        """
+        cf_name = self._get_coord_variable_name(cube, cell_measure)
+        while cf_name in self._dataset.variables:
+            cf_name = self._increment_name(cf_name)
+
+        # Derive the data dimension names for the coordinate.
+        cf_dimensions = [dimension_names[dim] for dim in
+                         cube.cell_measure_dims(cell_measure)]
+
+        # Get the values in a form which is valid for the file format.
+        data = self._ensure_valid_dtype(cell_measure.data, 'coordinate',
+                                        cell_measure)
+
+        # Create the CF-netCDF variable.
+        cf_var = self._dataset.createVariable(
+            cf_name, data.dtype.newbyteorder('='), cf_dimensions)
+
+        # Add the data to the CF-netCDF variable.
+        cf_var[:] = data
+
+        if cell_measure.units != 'unknown':
+            cf_var.units = str(cell_measure.units)
+
+        if cell_measure.standard_name is not None:
+            cf_var.standard_name = cell_measure.standard_name
+
+        if cell_measure.long_name is not None:
+            cf_var.long_name = cell_measure.long_name
+
+        # Add any other custom coordinate attributes.
+        for name in sorted(cell_measure.attributes):
+            value = cell_measure.attributes[name]
+
+            # Don't clobber existing attributes.
+            if not hasattr(cf_var, name):
+                cf_var.setncattr(name, value)
+
+        return cf_name
+
     def _create_cf_variable(self, cube, dimension_names, coord):
         """
         Create the associated CF-netCDF variable in the netCDF dataset for the
@@ -1318,7 +1585,7 @@ class Saver(object):
 
             # Don't clobber existing attributes.
             if not hasattr(cf_var, name):
-                setattr(cf_var, name, value)
+                cf_var.setncattr(name, value)
 
         return cf_name
 
@@ -1440,6 +1707,47 @@ class Saver(object):
                     cf_var_grid.scale_factor_at_central_meridian = (
                         cs.scale_factor_at_central_meridian)
 
+                # merc
+                elif isinstance(cs, iris.coord_systems.Mercator):
+                    if cs.ellipsoid:
+                        add_ellipsoid(cs.ellipsoid)
+                    cf_var_grid.longitude_of_projection_origin = (
+                        cs.longitude_of_projection_origin)
+                    # The Mercator class has implicit defaults for certain
+                    # parameters
+                    cf_var_grid.false_easting = 0.0
+                    cf_var_grid.false_northing = 0.0
+                    cf_var_grid.scale_factor_at_projection_origin = 1.0
+
+                # lcc
+                elif isinstance(cs, iris.coord_systems.LambertConformal):
+                    if cs.ellipsoid:
+                        add_ellipsoid(cs.ellipsoid)
+                    cf_var_grid.standard_parallel = cs.secant_latitudes
+                    cf_var_grid.latitude_of_projection_origin = cs.central_lat
+                    cf_var_grid.longitude_of_central_meridian = cs.central_lon
+                    cf_var_grid.false_easting = cs.false_easting
+                    cf_var_grid.false_northing = cs.false_northing
+
+                # stereo
+                elif isinstance(cs, iris.coord_systems.Stereographic):
+                    if cs.true_scale_lat is not None:
+                        warnings.warn('Stereographic coordinate systems with '
+                                      'true scale latitude specified are not '
+                                      'yet handled')
+                    else:
+                        if cs.ellipsoid:
+                            add_ellipsoid(cs.ellipsoid)
+                        cf_var_grid.longitude_of_projection_origin = (
+                            cs.central_lon)
+                        cf_var_grid.latitude_of_projection_origin = (
+                            cs.central_lat)
+                        cf_var_grid.false_easting = cs.false_easting
+                        cf_var_grid.false_northing = cs.false_northing
+                        # The Stereographic class has an implicit scale
+                        # factor
+                        cf_var_grid.scale_factor_at_projection_origin = 1.0
+
                 # osgb (a specific tmerc)
                 elif isinstance(cs, iris.coord_systems.OSGB):
                     warnings.warn('OSGB coordinate system not yet handled')
@@ -1555,7 +1863,7 @@ class Saver(object):
                       'global attribute.'.format(attr_name=attr_name)
                 warnings.warn(msg)
 
-            setattr(cf_var, attr_name, value)
+            cf_var.setncattr(attr_name, value)
 
         # Create the CF-netCDF data variable cell method attribute.
         cell_methods = self._create_cf_cell_methods(cube, dimension_names)
@@ -1783,4 +2091,4 @@ def _no_unlim_dep_warning():
            'deprecated, in favour of no automatic assignment. To switch '
            'to the new behaviour, set iris.FUTURE.netcdf_no_unlimited to '
            'True.')
-    warnings.warn(msg)
+    warn_deprecated(msg)
