@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2015, Met Office
+# (C) British Crown Copyright 2010 - 2016, Met Office
 #
 # This file is part of Iris.
 #
@@ -69,6 +69,7 @@ import iris.util
 # (And remove matplotlib as an iris.tests dependency.)
 try:
     import matplotlib
+    matplotlib.use('agg')
     import matplotlib.testing.compare as mcompare
     import matplotlib.pyplot as plt
 except ImportError:
@@ -84,11 +85,29 @@ else:
     GDAL_AVAILABLE = True
 
 try:
-    import gribapi
-except ImportError:
-    GRIB_AVAILABLE = False
-else:
+    import iris_grib
     GRIB_AVAILABLE = True
+    from iris_grib.message import GribMessage
+except ImportError:
+    try:
+        import gribapi
+        GRIB_AVAILABLE = True
+        from iris.fileformats.grib.message import GribMessage
+    except ImportError:
+        GRIB_AVAILABLE = False
+
+try:
+    import iris_sample_data
+except ImportError:
+    SAMPLE_DATA_AVAILABLE = False
+else:
+    SAMPLE_DATA_AVAILABLE = True
+
+try:
+    import nc_time_axis
+    NC_TIME_AXIS_AVAILABLE = True
+except ImportError:
+    NC_TIME_AXIS_AVAILABLE = False
 
 
 #: Basepath for test results.
@@ -115,13 +134,10 @@ logger = logging.getLogger('tests')
 # Whether to display matplotlib output to the screen.
 _DISPLAY_FIGURES = False
 
-if MPL_AVAILABLE:
-    if '-d' in sys.argv:
-        sys.argv.remove('-d')
-        plt.switch_backend('tkagg')
-        _DISPLAY_FIGURES = True
-    else:
-        plt.switch_backend('agg')
+if (MPL_AVAILABLE and '-d' in sys.argv):
+    sys.argv.remove('-d')
+    plt.switch_backend('tkagg')
+    _DISPLAY_FIGURES = True
 
 _DEFAULT_IMAGE_TOLERANCE = 10.0
 
@@ -181,18 +197,9 @@ def get_data_path(relative_path):
                     with open(data_path, 'wb') as fh:
                         fh.writelines(gz_fh)
 
-
     return data_path
 
 
-def get_result_path(relative_path):
-    """Returns the absolute path to a result file when given the relative path
-    as a string, or sequence of strings."""
-    if not isinstance(relative_path, six.string_types):
-        relative_path = os.path.join(*relative_path)
-    return os.path.abspath(os.path.join(_RESULT_PATH, relative_path))
-
-
 class IrisTest(unittest.TestCase):
     """A subclass of unittest.TestCase which provides Iris specific testing functionality."""
 
@@ -209,6 +216,17 @@ class IrisTest(unittest.TestCase):
                                                  'Reference', 'Test result', '', '', 0))
             self.fail("%s do not match: %s\n%s" % (type_comparison_name, reference_filename, diff))
 
+    @staticmethod
+    def get_result_path(relative_path):
+        """
+        Returns the absolute path to a result file when given the relative path
+        as a string, or sequence of strings.
+
+        """
+        if not isinstance(relative_path, six.string_types):
+            relative_path = os.path.join(*relative_path)
+        return os.path.abspath(os.path.join(_RESULT_PATH, relative_path))
+
     def result_path(self, basename=None, ext=''):
         """
         Return the full path to a test result, generated from the \
@@ -238,7 +256,7 @@ class IrisTest(unittest.TestCase):
                     break
         filename = basename + ext
 
-        result = os.path.join(get_result_path(''),
+        result = os.path.join(self.get_result_path(''),
                               sub_path.replace('test_', ''),
                               self.__class__.__name__.replace('Test_', ''),
                               filename)
@@ -251,7 +269,7 @@ class IrisTest(unittest.TestCase):
             cubes = [cubes]
         if reference_filename is None:
             reference_filename = self.result_path(None, 'cml')
-            reference_filename = [get_result_path(reference_filename)]
+            reference_filename = [self.get_result_path(reference_filename)]
         for i, cube in enumerate(cubes):
             fname = list(reference_filename)
             # don't want the ".cml" for the numpy data file
@@ -291,7 +309,7 @@ class IrisTest(unittest.TestCase):
         if reference_filename is None:
             reference_path = self.result_path(None, 'cdl')
         else:
-            reference_path = get_result_path(reference_filename)
+            reference_path = self.get_result_path(reference_filename)
 
         # Convert the netCDF file to CDL file format.
         cdl_filename = iris.util.create_temp_filename(suffix='.cdl')
@@ -358,7 +376,7 @@ class IrisTest(unittest.TestCase):
                                                 byteorder=False)
         else:
             xml = cubes.xml(checksum=checksum, order=False, byteorder=False)
-        reference_path = get_result_path(reference_filename)
+        reference_path = self.get_result_path(reference_filename)
         self._check_same(xml, reference_path)
 
     def assertTextFile(self, source_filename, reference_filename, desc="text file"):
@@ -372,7 +390,7 @@ class IrisTest(unittest.TestCase):
             self.fail("%s does not match reference file: %s\n%s" % (desc, reference_filename, diff))
 
     def assertCubeDataAlmostEqual(self, cube, reference_filename, *args, **kwargs):
-        reference_path = get_result_path(reference_filename)
+        reference_path = self.get_result_path(reference_filename)
         if self._check_reference_file(reference_path):
             kwargs.setdefault('err_msg', 'Reference file %s' % reference_path)
 
@@ -399,7 +417,7 @@ class IrisTest(unittest.TestCase):
                     np.save(reference_file, cube.data)
 
     def assertFilesEqual(self, test_filename, reference_filename):
-        reference_path = get_result_path(reference_filename)
+        reference_path = self.get_result_path(reference_filename)
         if self._check_reference_file(reference_path):
             fmt = 'test file {!r} does not match reference {!r}.'
             self.assertTrue(filecmp.cmp(test_filename, reference_path),
@@ -433,7 +451,7 @@ class IrisTest(unittest.TestCase):
         if reference_filename is None:
             reference_path = self.result_path(None, 'txt')
         else:
-            reference_path = get_result_path(reference_filename)
+            reference_path = self.get_result_path(reference_filename)
         self._check_same(string, reference_path,
                          type_comparison_name='Strings')
 
@@ -463,7 +481,7 @@ class IrisTest(unittest.TestCase):
         doc = xml.dom.minidom.Document()
         doc.appendChild(obj.xml_element(doc))
         pretty_xml = doc.toprettyxml(indent="  ")
-        reference_path = get_result_path(reference_filename)
+        reference_path = self.get_result_path(reference_filename)
         self._check_same(pretty_xml, reference_path,
                          type_comparison_name='XML')
 
@@ -738,6 +756,9 @@ class IrisTest(unittest.TestCase):
         return start_result
 
 
+get_result_path = IrisTest.get_result_path
+
+
 class GraphicsTest(IrisTest):
 
     def setUp(self):
@@ -753,6 +774,105 @@ class GraphicsTest(IrisTest):
             plt.close('all')
 
 
+class TestGribMessage(IrisTest):
+
+    def assertGribMessageContents(self, filename, contents):
+        """
+        Evaluate whether all messages in a GRIB2 file contain the provided
+        contents.
+
+        * filename (string)
+            The path on disk of an existing GRIB file
+
+        * contents
+            An iterable of GRIB message keys and expected values.
+
+        """
+        messages = GribMessage.messages_from_filename(filename)
+        for message in messages:
+            for element in contents:
+                section, key, val = element
+                self.assertEqual(message.sections[section][key], val)
+
+    def assertGribMessageDifference(self, filename1, filename2, diffs,
+                                    skip_keys=(), skip_sections=()):
+        """
+        Evaluate that the two messages only differ in the ways specified.
+
+        * filename[0|1] (string)
+            The path on disk of existing GRIB files
+
+        * diffs
+            An dictionary of GRIB message keys and expected diff values:
+            {key: (m1val, m2val),...} .
+
+        * skip_keys
+            An iterable of key names to ignore during comparison.
+
+        * skip_sections
+            An iterable of section numbers to ignore during comparison.
+
+        """
+        messages1 = list(GribMessage.messages_from_filename(filename1))
+        messages2 = list(GribMessage.messages_from_filename(filename2))
+        self.assertEqual(len(messages1), len(messages2))
+        for m1, m2 in zip(messages1, messages2):
+            m1_sect = set(m1.sections.keys())
+            m2_sect = set(m2.sections.keys())
+
+            for missing_section in (m1_sect ^ m2_sect):
+                what = ('introduced'
+                        if missing_section in m1_sect else 'removed')
+                # Assert that an introduced section is in the diffs.
+                self.assertIn(missing_section, skip_sections,
+                              msg='Section {} {}'.format(missing_section,
+                                                         what))
+
+            for section in (m1_sect & m2_sect):
+                # For each section, check that the differences are
+                # known diffs.
+                m1_keys = set(m1.sections[section]._keys)
+                m2_keys = set(m2.sections[section]._keys)
+
+                difference = m1_keys ^ m2_keys
+                unexpected_differences = difference - set(skip_keys)
+                if unexpected_differences:
+                    self.fail("There were keys in section {} which \n"
+                              "weren't in both messages and which weren't "
+                              "skipped.\n{}"
+                              "".format(section,
+                                        ', '.join(unexpected_differences)))
+
+                keys_to_compare = m1_keys & m2_keys - set(skip_keys)
+
+                for key in keys_to_compare:
+                    m1_value = m1.sections[section][key]
+                    m2_value = m2.sections[section][key]
+                    msg = '{} {} != {}'
+                    if key not in diffs:
+                        # We have a key which we expect to be the same for
+                        # both messages.
+                        if isinstance(m1_value, np.ndarray):
+                            # A large tolerance appears to be required for
+                            # gribapi 1.12, but not for 1.14.
+                            self.assertArrayAlmostEqual(m1_value, m2_value,
+                                                        decimal=2)
+                        else:
+                            self.assertEqual(m1_value, m2_value,
+                                             msg=msg.format(key, m1_value,
+                                                            m2_value))
+                    else:
+                        # We have a key which we expect to be different
+                        # for each message.
+                        self.assertEqual(m1_value, diffs[key][0],
+                                         msg=msg.format(key, m1_value,
+                                                         diffs[key][0]))
+
+                        self.assertEqual(m2_value, diffs[key][1],
+                                         msg=msg.format(key, m2_value,
+                                                        diffs[key][1]))
+
+
 def skip_data(fn):
     """
     Decorator to choose whether to run tests, based on the availability of
@@ -814,6 +934,16 @@ skip_grib = unittest.skipIf(not GRIB_AVAILABLE, 'Test(s) require "gribapi", '
                                                 'which is not available.')
 
 
+skip_sample_data = unittest.skipIf(not SAMPLE_DATA_AVAILABLE,
+                                   ('Test(s) require "iris_sample_data", '
+                                    'which is not available.'))
+
+
+skip_nc_time_axis = unittest.skipIf(
+    not NC_TIME_AXIS_AVAILABLE,
+    'Test(s) require "nc_time_axis", which is not available.')
+
+
 def no_warnings(func):
     """
     Provides a decorator to ensure that there are no warnings raised
